#!/usr/bin/python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

# __all__ = []

# import pkgutil
# import inspect

# for loader, name, is_pkg in pkgutil.walk_packages(__path__):
#     # print("YMK name {0}".format(name))
#     module = loader.find_module(name).load_module(name)
#
#     for name, value in inspect.getmembers(module):
#         if name.startswith('__'):
#             continue
#
#         globals()[name] = value
#         __all__.append(name)
# print("YMK __all__ {0}".format(__all__))
